from __future__ import annotations
from jaclang import *
import typing
from enum import Enum, auto
if typing.TYPE_CHECKING:
    from graph_walker import graph_walker
else:
    graph_walker, = jac_import('graph_walker', items={'graph_walker': None})
if typing.TYPE_CHECKING:
    from agents import Agents
else:
    Agents, = jac_import('agents', items={'Agents': None})

class list_agents(graph_walker, Walker):

    class __specs__(Obj):
        private: static[bool] = False

    @with_entry
    def on_agents(self, here: Agents) -> None:
        if (agents := here.get_all()):
            for agent in agents:
                Jac.report(agent.export())


from __future__ import annotations
from jaclang import *
import typing
from enum import Enum, auto
if typing.TYPE_CHECKING:
    from jivas.agent.core.graph_walker import graph_walker
else:
    graph_walker, = jac_import('jivas.agent.core.graph_walker', items={'graph_walker': None})

class interact_graph_walker(graph_walker, Walker):
    pass


from __future__ import annotations
from jaclang import *
import typing
from enum import Enum, auto
if typing.TYPE_CHECKING:
    from action import Action
else:
    Action, = jac_import('action', items={'Action': None})
if typing.TYPE_CHECKING:
    from jivas.agent.core.agent import Agent
else:
    Agent, = jac_import('jivas.agent.core.agent', items={'Agent': None})
if typing.TYPE_CHECKING:
    from jivas.agent.action.actions import Actions
else:
    Actions, = jac_import('jivas.agent.action.actions', items={'Actions': None})
if typing.TYPE_CHECKING:
    from interact_graph_walker import interact_graph_walker
else:
    interact_graph_walker, = jac_import('interact_graph_walker', items={'interact_graph_walker': None})

class healthcheck(interact_graph_walker, Walker):

    class __specs__(Obj):
        private: static[bool] = False

    @with_entry
    def on_agent(self, here: Agent) -> None:
        self.visit(here.refs().filter(Actions, None))

    @with_entry
    def on_actions(self, here: Actions) -> None:
        if (trace := here.healthcheck()):
            Jac.get_context().status = 200
            for action_label in trace.keys():
                if trace[action_label] == False:
                    Jac.get_context().status = 503
                    break
        else:
            Jac.get_context().status = 503
        Jac.report(trace)


from __future__ import annotations
from jaclang import *
import typing
from enum import Enum, auto
if typing.TYPE_CHECKING:
    import inspect
else:
    inspect, = jac_import('inspect', 'py')
if typing.TYPE_CHECKING:
    import logging
else:
    logging, = jac_import('logging', 'py')
if typing.TYPE_CHECKING:
    from logging import Logger
else:
    Logger, = jac_import('logging', 'py', items={'Logger': None})
if typing.TYPE_CHECKING:
    from jivas.agent.modules.agentlib.utils import Utils
else:
    Utils, = jac_import('jivas.agent.modules.agentlib.utils', 'py', items={'Utils': None})
if typing.TYPE_CHECKING:
    from typing import Optional, Type, Union
else:
    Optional, Type, Union = jac_import('typing', 'py', items={'Optional': None, 'Type': None, 'Union': None})
if typing.TYPE_CHECKING:
    from dataclasses import fields, MISSING
else:
    fields, MISSING = jac_import('dataclasses', 'py', items={'fields': None, 'MISSING': None})

class GraphNode(Node):
    id: str = field('')
    graph_children: bool = field(False)
    graph_self: bool = field(True)
    protected_attrs: list = field(gen=lambda: JacList(['id']))
    transient_attrs: list = field(gen=lambda: JacList(['__jac__', 'protected_attrs', 'transient_attrs', 'graph_children', 'graph_self', 'package_path']))
    _context: dict = field(gen=lambda: {})
    logger: static[Logger] = logging.getLogger(__name__)

    def __post_init__(self) -> None:
        if not self.id:
            self.id = jid(self)

    def get_type(self) -> str:
        return type(self).__name__

    def get_parent_type(self) -> str:
        return type(super()()).__name__

    def export(self, ignore_keys: list=JacList([]), modified_context: bool=False) -> dict:
        if modified_context:
            ignore_keys = ignore_keys + self.transient_attrs
            cls = type(self)
            default_values = {}
            for f in fields(self):
                if f.default is not MISSING:
                    default_values[f.name] = f.default
                elif f.default_factory is not MISSING:
                    default_values[f.name] = f.default_factory()
            node_export = Utils.export_to_dict(self, ignore_keys)
            return Utils.sanitize_dict_context(descriptor_data=node_export, action_data=default_values, keys_to_remove=ignore_keys)
        else:
            ignore_keys = ignore_keys + self.transient_attrs
            node_export = Utils.export_to_dict(self, ignore_keys)
            if isinstance(node_export['_context'], dict):
                node_export.update(node_export['_context'])
                del node_export['_context']
            return node_export

    def update(self, data: dict={}) -> GraphNode:
        if data:
            for attr in data.keys():
                if attr not in self.protected_attrs:
                    if hasattr(self, attr):
                        setattr(self, attr, data[attr])
                    else:
                        self._context[attr] = data[attr]
        self.postupdate()
        return self

    def postupdate(self) -> None:
        pass


from __future__ import annotations
from jaclang import *
import typing
from enum import Enum, auto
if typing.TYPE_CHECKING:
    import logging
else:
    logging, = jac_import('logging', 'py')
if typing.TYPE_CHECKING:
    import traceback
else:
    traceback, = jac_import('traceback', 'py')
if typing.TYPE_CHECKING:
    from logging import Logger
else:
    Logger, = jac_import('logging', 'py', items={'Logger': None})
if typing.TYPE_CHECKING:
    from jivas.agent.modules.agentlib.utils import Utils
else:
    Utils, = jac_import('jivas.agent.modules.agentlib.utils', 'py', items={'Utils': None})
if typing.TYPE_CHECKING:
    from jivas.agent.core import graph_node, purge
else:
    graph_node, purge = jac_import('jivas.agent.core', items={'graph_node': None, 'purge': None})
if typing.TYPE_CHECKING:
    from graph_node import GraphNode
else:
    GraphNode, = jac_import('graph_node', items={'GraphNode': None})
if typing.TYPE_CHECKING:
    from jivas.agent.action.action import Action
else:
    Action, = jac_import('jivas.agent.action.action', items={'Action': None})
if typing.TYPE_CHECKING:
    from jivas.agent.action.interact_action import InteractAction
else:
    InteractAction, = jac_import('jivas.agent.action.interact_action', items={'InteractAction': None})
if typing.TYPE_CHECKING:
    from jivas.agent.action.actions import Actions
else:
    Actions, = jac_import('jivas.agent.action.actions', items={'Actions': None})
if typing.TYPE_CHECKING:
    from jivas.agent.memory.memory import Memory
else:
    Memory, = jac_import('jivas.agent.memory.memory', items={'Memory': None})

class Agent(GraphNode, Node):
    logger: static[Logger] = logging.getLogger(__name__)
    published: bool = field(True)
    name: str = field('')
    description: str = field('')
    descriptor: str = field('')
    jpr_api_key: str = field('')
    agent_logging: bool = field(True)
    message_limit: int = field(1024)
    flood_block_time: int = field(300)
    window_time: int = field(20)
    flood_threshold: int = field(4)
    frame_size: int = field(10)
    meta: dict = field(gen=lambda: {})
    tts_action: str = field('ElevenlabsTTSAction')
    stt_action: str = field('DeepgramSTTAction')
    vector_store_action: str = field('TypesenseVectorStoreAction')

    def __post_init__(self) -> None:
        super().__post_init__()
        self.protected_attrs += JacList(['id', 'actions'])

    def get_memory(self) -> None:
        memory_node = Utils.node_obj(self.refs().filter(Memory, None))
        if not memory_node:
            self.connect((memory_node := Memory()))
            self.logger.debug('memory node created')
        return memory_node

    def get_actions(self) -> None:
        actions_node = Utils.node_obj(self.refs().filter(Actions, None))
        if not actions_node:
            self.connect((actions_node := Actions()))
            self.logger.debug('actions node created')
        return actions_node

    def get_tts_action(self) -> None:
        if (tts_action_node := self.get_actions().get(action_label=self.tts_action)):
            if tts_action_node.enabled:
                return tts_action_node
        return None

    def get_stt_action(self) -> None:
        if (stt_action_node := self.get_actions().get(action_label=self.stt_action)):
            if stt_action_node.enabled:
                return stt_action_node
        return None

    def get_vector_store_action(self) -> None:
        if (vector_store_action_node := self.get_actions().get(action_label=self.vector_store_action)):
            if vector_store_action_node.enabled:
                return vector_store_action_node
        return None

    def update(self, data: dict={}, with_actions: bool=False) -> Agent:
        agent_node = super().update(data=data)
        if with_actions and len(data.get('actions', JacList([]))) > 0:
            self.get_actions().install_actions(self.id, data['actions'], self.jpr_api_key)
        return agent_node

    def is_logging(self) -> None:
        return self.agent_logging

    def set_logging(self, agent_logging: bool) -> None:
        self.agent_logging = agent_logging

    def export_descriptor(self, file_path: str='', modified_context: bool=False) -> dict:
        try:
            self.logger.debug(f'exporting agent: {self.name}')
            agent_data = self.export(JacList([]), modified_context)
            agent_actions = JacList([])
            agent_actions = self.spawn(_export_actions(modified_context=modified_context)).action_nodes
            agent_data = {**agent_data, **{'actions': agent_actions}}
            if file_path:
                Utils.dump_yaml_file(file_path=file_path, data=agent_data)
            else:
                Utils.dump_yaml_file(file_path=self.descriptor, data=agent_data)
            return agent_data
        except Exception as e:
            self.logger.error(f'an exception occurred, {traceback.format_exc()}')
        return {}

class _export_actions(Walker):
    transient_export_attrs: list = field(gen=lambda: JacList(['_package']))
    action_nodes: list = field(gen=lambda: JacList([]))
    node_index: dict = field(gen=lambda: {})
    modified_context: bool = field(False)

    class __specs__(Obj):
        private: static[bool] = True

    @with_entry
    def on_agent(self, here: Agent) -> None:
        self.visit(here.refs().filter(Actions, None))

    @with_entry
    def on_actions(self, here: Actions) -> None:
        self.visit(here.refs().filter(Action, None))

    @with_entry
    def on_action(self, here: Action) -> None:
        if here.label != 'ExitInteractAction':
            children = JacList([])
            if isinstance(here, InteractAction):
                child_nodes = here.get_children()
                for child in child_nodes:
                    children.append(child.id)
            self.node_index.update({here.id: {'action': here._package['name'], 'context': here.export(self.transient_export_attrs, self.modified_context), 'children': children}})
            self.ignore(here)
        self.visit(here.refs().filter(Action, None))

    @with_exit
    def on_exit(self, here) -> None:
        if self.node_index:
            node_keys = list(self.node_index.keys())
            node_keys.reverse()
            for key in node_keys:
                resolved_nodes = JacList([])
                for child_id in self.node_index[key]['children']:
                    resolved_nodes.append(self.node_index[child_id])
                    self.node_index.pop(child_id)
                if resolved_nodes:
                    self.node_index[key]['children'] = resolved_nodes
                else:
                    self.node_index[key].pop('children')
            self.action_nodes = list(self.node_index.values())


from __future__ import annotations
from jaclang import *
import typing
from enum import Enum, auto
if typing.TYPE_CHECKING:
    import os
else:
    os, = jac_import('os', 'py')
if typing.TYPE_CHECKING:
    import io
else:
    io, = jac_import('io', 'py')
if typing.TYPE_CHECKING:
    import yaml
else:
    yaml, = jac_import('yaml', 'py')
if typing.TYPE_CHECKING:
    import logging
else:
    logging, = jac_import('logging', 'py')
if typing.TYPE_CHECKING:
    import traceback
else:
    traceback, = jac_import('traceback', 'py')
if typing.TYPE_CHECKING:
    from logging import Logger
else:
    Logger, = jac_import('logging', 'py', items={'Logger': None})
if typing.TYPE_CHECKING:
    from jivas.agent.modules.agentlib.utils import Utils, jvdata_file_interface
else:
    Utils, jvdata_file_interface = jac_import('jivas.agent.modules.agentlib.utils', 'py', items={'Utils': None, 'jvdata_file_interface': None})
if typing.TYPE_CHECKING:
    from jivas.agent.core.agent import Agent
else:
    Agent, = jac_import('jivas.agent.core.agent', items={'Agent': None})
if typing.TYPE_CHECKING:
    from jivas.agent.action.actions import Actions
else:
    Actions, = jac_import('jivas.agent.action.actions', items={'Actions': None})
if typing.TYPE_CHECKING:
    from jivas.agent.core.graph_walker import graph_walker
else:
    graph_walker, = jac_import('jivas.agent.core.graph_walker', items={'graph_walker': None})
if typing.TYPE_CHECKING:
    from jivas.agent.core.import_agent import import_agent
else:
    import_agent, = jac_import('jivas.agent.core.import_agent', items={'import_agent': None})

class install_action(graph_walker, Walker):
    reporting: bool = field(False)
    override_action: bool = field(True)
    package_name: str = field('')
    version: str = field('')
    logger: static[Logger] = logging.getLogger(__name__)

    class __specs__(Obj):
        private: static[bool] = False

    @with_entry
    def on_agent(self, here: Agent) -> None:
        descriptor_data = None
        try:
            file_bytes = jvdata_file_interface.get_file(here.descriptor)
            file = io.BytesIO(file_bytes)
            descriptor_data = yaml.safe_load(file)
            if not descriptor_data:
                self.logger.error(f'unable to load descriptor: {here.descriptor}')
                return None
        except Exception as e:
            self.logger.error(f'an exception occurred, {traceback.format_exc()}')
            return None
        jpr_api_key = descriptor_data.get('jpr_api_key')
        dep_action_info = here.get_actions().get_action_info(namespace_package_name=self.package_name, version=self.version, jpr_api_key=jpr_api_key)
        if dep_action_info:
            action_info = {'action': dep_action_info.get('name'), 'context': {'version': dep_action_info.get('version'), 'enabled': True}}
            action_exists = False
            for action in descriptor_data['actions']:
                if action['action'] == self.package_name:
                    if self.override_action:
                        action['context'] = action_info['context']
                    else:
                        action['context'].update(action_info['context'])
                    action_exists = True
            if not action_exists:
                descriptor_data['actions'].append(action_info)
            self.import_from_descriptor(descriptor_data)

    def import_from_descriptor(self, descriptor: dict) -> Agent:
        if descriptor.get('id', None):
            try:
                agent_node = jobj(id=descriptor['id'])
            except Exception as e:
                self.logger.error(f'an exception occurred, {traceback.format_exc()}')
                return None
        else:
            agent_name = descriptor.get('name')
            if (_node := agents_node.get_by_name(agent_name)):
                agent_node = _node
            elif (agent_node := Agent(name=agent_name, description=descriptor.get('description', ''))):
                if not agents_node.get_by_id(agent_node.id):
                    agents_node.connect(agent_node)
                self.logger.info(f'agent created: {agent_node.name}')
            else:
                self.logger.error(f'unable to create agent: {agent_name}')
        if agent_node:
            descriptor_root = Utils.get_descriptor_root()
            agent_node.descriptor = f'{descriptor_root}/{agent_node.id}.yaml'
            agent_node.get_memory()
            agent_node.get_actions()
            if (agent_node := agent_node.update(data=descriptor, with_actions=True)):
                agent_node.export_descriptor()
                return agent_node
        return None


from __future__ import annotations
from jaclang import *
import typing
from enum import Enum, auto
if typing.TYPE_CHECKING:
    import os
else:
    os, = jac_import('os', 'py')
if typing.TYPE_CHECKING:
    import logging
else:
    logging, = jac_import('logging', 'py')
if typing.TYPE_CHECKING:
    import traceback
else:
    traceback, = jac_import('traceback', 'py')
if typing.TYPE_CHECKING:
    import json
else:
    json, = jac_import('json', 'py')
if typing.TYPE_CHECKING:
    from logging import Logger
else:
    Logger, = jac_import('logging', 'py', items={'Logger': None})
if typing.TYPE_CHECKING:
    from datetime import datetime, timedelta
else:
    datetime, timedelta = jac_import('datetime', 'py', items={'datetime': None, 'timedelta': None})
if typing.TYPE_CHECKING:
    from jivas.agent.core.graph_walker import graph_walker
else:
    graph_walker, = jac_import('jivas.agent.core.graph_walker', items={'graph_walker': None})
if typing.TYPE_CHECKING:
    from jivas.agent.core.agent import Agent
else:
    Agent, = jac_import('jivas.agent.core.agent', items={'Agent': None})
if typing.TYPE_CHECKING:
    from jac_cloud.core.architype import NodeAnchor
else:
    NodeAnchor, = jac_import('jac_cloud.core.architype', 'py', items={'NodeAnchor': None})

class get_interaction_logs(graph_walker, Walker):
    start_date: str = field('')
    end_date: str = field('')
    session_id: str = field('')
    channel: str = field('')
    timezone: str = field('UTC')
    logger: static[Logger] = logging.getLogger(__name__)

    class __specs__(Obj):
        private: static[bool] = False

    @with_entry
    def on_agent(self, here: Agent) -> None:
        try:
            start = datetime.strptime(f'{self.start_date}T00:00:00+00:00', '%Y-%m-%dT%H:%M:%S%z')
            end = datetime.strptime(f'{self.end_date}T00:00:00+00:00', '%Y-%m-%dT%H:%M:%S%z')
            end = end + timedelta(days=1) - timedelta(milliseconds=1)
            collection = NodeAnchor.Collection.get_collection('interactions')
            match_criteria = {'agent_id': self.agent_id, 'time_stamp': {'$gte': start.isoformat(), '$lte': end.isoformat()}}
            if self.session_id:
                match_criteria['session_id'] = self.session_id
            if self.channel:
                match_criteria['channel'] = self.channel
            pipeline = JacList([{'$match': match_criteria}, {'$sort': {'time_stamp': 1}}])
            result = list(collection.aggregate(pipeline))
            Jac.report({'total': len(result), 'data': json.loads(json.dumps(result, default=str))})
        except Exception as e:
            self.logger.error(f'An error occurred while fetching interaction logs: {traceback.format_exc()}')
            Jac.report({'error': str(e)})